"""Conversation context management for improved detection accuracy."""

from fastlangml.context.conversation import ConversationContext, ConversationTurn

__all__ = [
    "ConversationContext",
    "ConversationTurn",
]
