"""User-defined word hints for language detection."""

from fastlangml.hints.dictionary import HintDictionary
from fastlangml.hints.persistence import HintPersistence

__all__ = [
    "HintDictionary",
    "HintPersistence",
]
